@@ -16,21 +16,18 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/ext/transport/chttp2/transport/frame_goaway.h"
 
-#include <string.h>
-
-#include "absl/base/attributes.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_format.h"
-#include "absl/strings/string_view.h"
-
 #include <grpc/slice_buffer.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
+#include <string.h>
 
+#include "absl/base/attributes.h"
+#include "absl/log/check.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
 #include "src/core/ext/transport/chttp2/transport/internal.h"
 
 void grpc_chttp2_goaway_parser_init(grpc_chttp2_goaway_parser* p) {
@@ -75,7 +72,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->last_stream_id = (static_cast<uint32_t>(*cur)) << 24;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_LSI1:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_LSI1;
@@ -83,7 +80,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->last_stream_id |= (static_cast<uint32_t>(*cur)) << 16;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_LSI2:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_LSI2;
@@ -91,7 +88,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->last_stream_id |= (static_cast<uint32_t>(*cur)) << 8;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_LSI3:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_LSI3;
@@ -99,7 +96,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->last_stream_id |= (static_cast<uint32_t>(*cur));
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_ERR0:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_ERR0;
@@ -107,7 +104,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->error_code = (static_cast<uint32_t>(*cur)) << 24;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_ERR1:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_ERR1;
@@ -115,7 +112,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->error_code |= (static_cast<uint32_t>(*cur)) << 16;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_ERR2:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_ERR2;
@@ -123,7 +120,7 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->error_code |= (static_cast<uint32_t>(*cur)) << 8;
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_ERR3:
       if (cur == end) {
         p->state = GRPC_CHTTP2_GOAWAY_ERR3;
@@ -131,16 +128,21 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
       }
       p->error_code |= (static_cast<uint32_t>(*cur));
       ++cur;
-      ABSL_FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case GRPC_CHTTP2_GOAWAY_DEBUG:
       if (end != cur) {
         memcpy(p->debug_data + p->debug_pos, cur,
                static_cast<size_t>(end - cur));
       }
-      GPR_ASSERT((size_t)(end - cur) < UINT32_MAX - p->debug_pos);
+      CHECK((size_t)(end - cur) < UINT32_MAX - p->debug_pos);
       p->debug_pos += static_cast<uint32_t>(end - cur);
       p->state = GRPC_CHTTP2_GOAWAY_DEBUG;
       if (is_last) {
+        t->http2_ztrace_collector.Append([p]() {
+          return grpc_core::H2GoAwayTrace<true>{
+              p->last_stream_id, p->error_code,
+              std::string(absl::string_view(p->debug_data, p->debug_length))};
+        });
         grpc_chttp2_add_incoming_goaway(
             t, p->error_code, p->last_stream_id,
             absl::string_view(p->debug_data, p->debug_length));
@@ -152,15 +154,22 @@ grpc_error_handle grpc_chttp2_goaway_parser_parse(void* parser,
   GPR_UNREACHABLE_CODE(return GRPC_ERROR_CREATE("Should never reach here"));
 }
 
-void grpc_chttp2_goaway_append(uint32_t last_stream_id, uint32_t error_code,
-                               const grpc_slice& debug_data,
-                               grpc_slice_buffer* slice_buffer) {
+void grpc_chttp2_goaway_append(
+    uint32_t last_stream_id, uint32_t error_code, const grpc_slice& debug_data,
+    grpc_slice_buffer* slice_buffer,
+    grpc_core::Http2ZTraceCollector* ztrace_collector) {
   grpc_slice header = GRPC_SLICE_MALLOC(9 + 4 + 4);
   uint8_t* p = GRPC_SLICE_START_PTR(header);
   uint32_t frame_length;
-  GPR_ASSERT(GRPC_SLICE_LENGTH(debug_data) < UINT32_MAX - 4 - 4);
+  CHECK(GRPC_SLICE_LENGTH(debug_data) < UINT32_MAX - 4 - 4);
   frame_length = 4 + 4 + static_cast<uint32_t> GRPC_SLICE_LENGTH(debug_data);
 
+  ztrace_collector->Append([last_stream_id, error_code, debug_data]() {
+    return grpc_core::H2GoAwayTrace<false>{
+        last_stream_id, error_code,
+        std::string(grpc_core::StringViewFromSlice(debug_data))};
+  });
+
   // frame header: length
   *p++ = static_cast<uint8_t>(frame_length >> 16);
   *p++ = static_cast<uint8_t>(frame_length >> 8);
@@ -184,7 +193,7 @@ void grpc_chttp2_goaway_append(uint32_t last_stream_id, uint32_t error_code,
   *p++ = static_cast<uint8_t>(error_code >> 16);
   *p++ = static_cast<uint8_t>(error_code >> 8);
   *p++ = static_cast<uint8_t>(error_code);
-  GPR_ASSERT(p == GRPC_SLICE_END_PTR(header));
+  CHECK(p == GRPC_SLICE_END_PTR(header));
   grpc_slice_buffer_add(slice_buffer, header);
   grpc_slice_buffer_add(slice_buffer, debug_data);
 }
