@@ -26,6 +26,7 @@
 #include <cstdint>
 #include <cstdio>
 #include <cstdlib>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -35,7 +36,6 @@
 #include "absl/functional/function_ref.h"
 #include "absl/log/log.h"
 #include "absl/strings/string_view.h"
-#include "absl/types/optional.h"
 #include "src/core/util/per_cpu.h"
 #include "src/core/util/sync.h"
 
@@ -47,7 +47,7 @@ namespace latent_see {
 struct Metadata {
   const char* file;
   int line;
-  const char* name;
+  absl::string_view name;
 };
 
 enum class EventType : uint8_t { kBegin, kEnd, kFlowStart, kFlowEnd, kMark };
@@ -119,38 +119,23 @@ class Log {
   static Bin* CurrentThreadBin() { return bin_; }
 
   GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION static Log& Get() {
-    static Log* log = []() {
-      atexit([] {
-        auto json = log->TryGenerateJson();
-        if (!json.has_value()) {
-          LOG(INFO) << "Failed to generate latent_see.json (contention with "
-                       "another writer)";
-          return;
-        }
-        if (log->stats_flusher_ != nullptr) {
-          log->stats_flusher_(*json);
-          return;
-        }
-        LOG(INFO) << "Writing latent_see.json in " << get_current_dir_name();
-        FILE* f = fopen("latent_see.json", "w");
-        if (f == nullptr) return;
-        fprintf(f, "%s", json->c_str());
-        fclose(f);
-      });
-      return new Log();
-    }();
+    static Log* log = new Log();
     return *log;
   }
 
   void TryPullEventsAndFlush(
       absl::FunctionRef<void(absl::Span<const RecordedEvent>)> callback);
-  absl::optional<std::string> TryGenerateJson();
+  std::optional<std::string> TryGenerateJson();
 
   void OverrideStatsFlusher(
       absl::AnyInvocable<void(absl::string_view)> stats_exporter) {
     stats_flusher_ = std::move(stats_exporter);
   }
 
+  // Install an atexit callback that will log to latent_see.json in the working
+  // directory
+  static void InstallAtExitHandler();
+
  private:
   Log() = default;
 
@@ -274,6 +259,9 @@ GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION auto Promise(const Metadata* md_poll,
 
 }  // namespace latent_see
 }  // namespace grpc_core
+#define GRPC_LATENT_SEE_SYMBOL2(name, line) name##_##line
+#define GRPC_LATENT_SEE_SYMBOL1(name, line) GRPC_LATENT_SEE_SYMBOL2(name, line)
+#define GRPC_LATENT_SEE_SYMBOL(name) GRPC_LATENT_SEE_SYMBOL1(name, __LINE__)
 #define GRPC_LATENT_SEE_METADATA(name)                                     \
   []() {                                                                   \
     static grpc_core::latent_see::Metadata metadata = {__FILE__, __LINE__, \
@@ -283,23 +271,26 @@ GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION auto Promise(const Metadata* md_poll,
 // Parent scope: logs a begin and end event, and flushes the thread log on scope
 // exit. Because the flush takes some time it's better to place one parent scope
 // at the top of the stack, and use lighter weight scopes within it.
-#define GRPC_LATENT_SEE_PARENT_SCOPE(name)                       \
-  grpc_core::latent_see::ParentScope latent_see_scope##__LINE__( \
+#define GRPC_LATENT_SEE_PARENT_SCOPE(name)                                     \
+  grpc_core::latent_see::ParentScope GRPC_LATENT_SEE_SYMBOL(latent_see_scope)( \
       GRPC_LATENT_SEE_METADATA(name))
 // Inner scope: logs a begin and end event. Lighter weight than parent scope,
 // but does not flush the thread state - so should only be enclosed by a parent
 // scope.
-#define GRPC_LATENT_SEE_INNER_SCOPE(name)                       \
-  grpc_core::latent_see::InnerScope latent_see_scope##__LINE__( \
+#define GRPC_LATENT_SEE_INNER_SCOPE(name)                                     \
+  grpc_core::latent_see::InnerScope GRPC_LATENT_SEE_SYMBOL(latent_see_scope)( \
       GRPC_LATENT_SEE_METADATA(name))
 // Mark: logs a single event.
 // This is not flushed automatically, and so should only be used within a parent
 // scope.
 #define GRPC_LATENT_SEE_MARK(name) \
   grpc_core::latent_see::Mark(GRPC_LATENT_SEE_METADATA(name))
-#define GRPC_LATENT_SEE_PROMISE(name, promise)                           \
-  grpc_core::latent_see::Promise(GRPC_LATENT_SEE_METADATA("Poll:" name), \
-                                 GRPC_LATENT_SEE_METADATA(name), promise)
+#define GRPC_LATENT_SEE_PROMISE(name, promise)                                 \
+  grpc_core::latent_see::Promise(GRPC_LATENT_SEE_METADATA("Poll:" name),       \
+                                 GRPC_LATENT_SEE_METADATA(name), [&]() {       \
+                                   GRPC_LATENT_SEE_INNER_SCOPE("Setup:" name); \
+                                   return promise;                             \
+                                 }())
 #else  // !def(GRPC_ENABLE_LATENT_SEE)
 namespace grpc_core {
 namespace latent_see {
